/*! Copyright (c) 2011 by Jonas Mosbech - https://github.com/jmosbech/StickyTableHeaders 
    MIT license info: https://github.com/jmosbech/StickyTableHeaders/blob/master/license.txt */

(function ($) {
	$.StickyTableHeaders = function (el, options) {
		// To avoid scope issues, use 'base' instead of 'this'
		// to reference this class from internal events and functions.
		var base = this;

		// Access to jQuery and DOM versions of element
		base.$el = $(el);
		base.el = el;

		// Cache DOM refs for performance reasons
		base.$window = $(window);
		base.$clonedHeader = null;
		base.$originalHeader = null;

		// Add a reverse reference to the DOM object
		base.$el.data('StickyTableHeaders', base);

		base.init = function () {
			base.options = $.extend({}, $.StickyTableHeaders.defaultOptions, options);

			base.$el.each(function () {
				var $this = $(this), kwn='';

				// remove padding on <table> to fix issue #7
				$this.css('padding', 0);
				
				// myDBR specific keepwithnext
				kwn = $this.hasClass('kwn') ? ' kwn' : '';
				$this.wrap('<div class="divTableWithFloatingHeader'+kwn+'"></div>');

				// In case the element is an inline element
				$this.parent().css('inline', $this.css('inline'));
				base.$originalHeader = $('thead:first', this);
				base.$clonedHeader = base.$originalHeader.clone();

				base.$clonedHeader.addClass('tableFloatingHeader');
				base.$clonedHeader.css({
					'position': 'fixed',
					'top': 0,
					'left': $this.css('margin-left'),
					'display': 'none'
				});

				base.$originalHeader.addClass('tableFloatingHeaderOriginal');

				base.$originalHeader.before(base.$clonedHeader);

				// enabling support for jquery.tablesorter plugin
				// forward clicks on clone to original
				$('th', base.$clonedHeader).click(function(e){
					var index = $('th', base.$clonedHeader).index(this);
					$('th', base.$originalHeader).eq(index).click();
				});
				$this.bind('sortEnd', base.updateCloneFromOriginal );
			});

			base.updateTableHeaders();
			base.$window.scroll(base.updateTableHeaders);
			base.$window.resize(base.updateTableHeaders);
		};

		base.width = function( el ) {
			var width = 0;
			/* JQuery width seems to have issues */
			if (window.getComputedStyle) {
				if (window.getComputedStyle(el[0],"").getPropertyValue) {
					width = parseFloat(window.getComputedStyle(el[0],"").getPropertyValue("width"));
				} else {
					width = parseFloat(window.getComputedStyle(el[0],"").width);
				}
			}
			if ((width==undefined) || (width==0) || isNaN(width)) {
				width=el.width();
			}
			return width;
		};
		
		base.updateTableHeaders = function () {
			base.$el.each(function () {
				var $this = $(this);

				var fixedHeaderHeight = isNaN(base.options.fixedOffset) ? base.options.fixedOffset.height() : base.options.fixedOffset;

				var offset = $this.offset();
				var caption_height = this.caption ? $(this.caption).height() : 0;
				var margin_bottom = parseInt($this.css('margin-bottom'),10);
				var scrollTop = base.$window.scrollTop() + fixedHeaderHeight;
				var scrollLeft = base.$window.scrollLeft();

				if ((scrollTop > offset.top+caption_height) && (scrollTop < offset.top + $this.height() - margin_bottom)) {
					base.$clonedHeader.css({
						'top': fixedHeaderHeight,
						'margin-top': 0,
						'margin-left': 0,
						'margin-right': 0,
						'left': offset.left - scrollLeft,
						'display': 'block'
					});

					base.updateCloneFromOriginal();
				}
				else {
					base.$clonedHeader.css('display', 'none');
				}
			});
		};

		base.updateCloneFromOriginal = function () {
			// Copy cell widths and classes from original header
			$('th', base.$clonedHeader).each(function (index) {
				var $this = $(this);
				var origCell = $('th', base.$originalHeader).eq(index);
				$this.removeClass().addClass(origCell.attr('class'));
				
				// $this.css('width', origCell.width());
				$this.css('width', base.width(origCell));
			});

			// Copy row width from whole table
			// base.$clonedHeader.css('width', base.$originalHeader.width());
			base.$clonedHeader.css('width', base.width(base.$originalHeader));
			// if (!$.browser.opera) {
				// base.$clonedHeader.width('100%');
			// }
		};

		// Run initializer
		base.init();
	};

	$.StickyTableHeaders.defaultOptions = {
		fixedOffset: 0
	};

	$.fn.stickyTableHeaders = function (options) {
		return this.each(function () {
			(new $.StickyTableHeaders(this, options));
		});
	};

})(jQuery);

